@@ -28,16 +28,17 @@ use constant COLOR   => ($ENV{APACHE_TEST_COLOR} && -t STDOUT) ? 1 : 0;
 use constant DEFAULT_PORT => 8529;
 
 use constant IS_MOD_PERL_2       =>
-    eval { require mod_perl && $mod_perl::VERSION >= 1.99 } || 0;
+    eval { require mod_perl2 } || 0;
 
 use constant IS_MOD_PERL_2_BUILD => IS_MOD_PERL_2 &&
-    require Apache::Build && Apache::Build::IS_MOD_PERL_BUILD();
+    require Apache2::Build && Apache2::Build::IS_MOD_PERL_BUILD();
 
 use constant IS_APACHE_TEST_BUILD =>
     grep { -e "$_/lib/Apache/TestConfig.pm" } qw(Apache-Test . ..);
 
 use constant CUSTOM_CONFIG_FILE => 'Apache/TestConfigData.pm';
 
+use lib ();
 use File::Copy ();
 use File::Find qw(finddepth);
 use File::Basename qw(dirname);
@@ -85,7 +86,7 @@ my %vars_to_env = (
    sbindir         => 'Apache sbin/ dir (default is apxs -q SBINDIR)',
    httpd           => 'server to use for testing (default is $bindir/httpd)',
    target          => 'name of server binary (default is apxs -q TARGET)',
-   apxs            => 'location of apxs (default is from Apache::BuildConfig)',
+   apxs            => 'location of apxs (default is from Apache2::BuildConfig)',
    startup_timeout => 'seconds to wait for the server to start (default is 60)',
    httpd_conf      => 'inherit config from this file (default is apxs derived)',
    httpd_conf_extra=> 'inherit additional config from this file',
@@ -171,15 +172,16 @@ sub passenv_makestr {
 
 sub server { shift->{server} }
 
-sub modperl_2_inc_fixup {
-    (IS_MOD_PERL_2 && !IS_MOD_PERL_2_BUILD) ? "use Apache2;\n" : '';
-}
-
 sub modperl_build_config {
+
+    # we don't want to get mp2 preconfigured data in order to be able
+    # to get the interactive tests running.
+    return undef if $ENV{APACHE_TEST_INTERACTIVE_CONFIG_TEST};
+
     eval {
-        require Apache::Build;
+        require Apache2::Build;
     } or return undef;
-    return Apache::Build->build_config;
+    return Apache2::Build->build_config;
 }
 
 sub new_test_server {
@@ -384,6 +386,9 @@ sub httpd_config {
         # so it never returns
         $self->custom_config_first_time($self->{vars});
     }
+    else {
+        debug "Using httpd: $vars->{httpd}";
+    }
 
     # if we have gotten that far we know at least about the location
     # of httpd and or apxs, so let's save it if we haven't saved any
@@ -1204,6 +1209,18 @@ sub parse_vhost {
     };
 }
 
+sub find_and_load_module {
+    my ($self, $name) = @_;
+    my $mod_path = $self->find_apache_module($name) or return;
+    my ($sym) = $name =~ m/mod_(\w+)\./;
+
+    if ($mod_path && -e $mod_path) {
+        $self->preamble(IfModule => "!mod_$sym.c",
+                        qq{LoadModule ${sym}_module "$mod_path"\n});
+    }
+    return 1;
+}
+
 sub replace_vhost_modules {
     my $self = shift;
 
@@ -1257,11 +1274,7 @@ sub generate_types_config {
 
     # handle the case when mod_mime is built as a shared object
     # but wasn't included in the system-wide httpd.conf
-    my $mod_mime = $self->find_apache_module('mod_mime.so');
-    if ($mod_mime && -e $mod_mime) {
-        $self->preamble(IfModule => '!mod_mime.c',
-                        qq{LoadModule mime_module "$mod_mime"\n});
-    }
+    $self->find_and_load_module('mod_mime.so');
 
     unless ($self->{inherit_config}->{TypesConfig}) {
         my $types = catfile $self->{vars}->{t_conf}, 'mime.types';
@@ -1355,25 +1368,21 @@ sub generate_extra_conf {
         push @extra_conf, $generated;
     }
 
-    # if at least one .in file was modified or the derivative is
-    # missing, regenerate them all (so information like assigned port
-    # numbers will be correct)
-    if ($self->extra_conf_files_needing_update) {
-        for my $file (@conf_files) {
-            local $Apache::TestConfig::File = $file;
+    # regenerate .conf files
+    for my $file (@conf_files) {
+        local $Apache::TestConfig::File = $file;
 
-            my $in = Symbol::gensym();
-            open($in, $file) or next;
+        my $in = Symbol::gensym();
+        open($in, $file) or next;
 
-            (my $generated = $file) =~ s/\.in$//;
-            my $out = $self->genfile($generated, $file);
-            $self->replace_vars($in, $out);
+        (my $generated = $file) =~ s/\.in$//;
+        my $out = $self->genfile($generated, $file);
+        $self->replace_vars($in, $out);
 
-            close $in;
-            close $out;
+        close $in;
+        close $out;
 
-            $self->check_vars;
-        }
+        $self->check_vars;
     }
 
     #we changed order to give ssl the first port after DEFAULT_PORT
@@ -1513,6 +1522,8 @@ sub generate_httpd_conf {
 
     my $out = $self->genfile($conf_file);
 
+    $self->find_and_load_module('mod_alias.so');
+
     $self->preamble_run($out);
 
     for my $name (qw(user group)) { #win32/cygwin do not support
@@ -1534,14 +1545,6 @@ sub generate_httpd_conf {
 
     # handle the case when mod_alias is built as a shared object
     # but wasn't included in the system-wide httpd.conf
-    my $mod_alias = $self->find_apache_module('mod_alias.so');
-    if ($mod_alias && -e $mod_alias) {
-        print $out <<EOF;
-<IfModule !mod_alias.c>
-    LoadModule alias_module "$mod_alias"
-</IfModule>
-EOF
-    }
 
     print $out "<IfModule mod_alias.c>\n";
     for (keys %aliases) {
@@ -1637,7 +1640,13 @@ sub which {
 
     return undef unless $program;
 
-    for my $base (map { catfile($_, $program) } File::Spec->path()) {
+    my @dirs = File::Spec->path();
+
+    require Config;
+    my $perl_bin = $Config::Config{bin} || '';
+    push @dirs, $perl_bin if $perl_bin and -d $perl_bin;
+
+    for my $base (map { catfile $_, $program } @dirs) {
         if ($ENV{HOME} and not WIN32) {
             # only works on Unix, but that's normal:
             # on Win32 the shell doesn't have special treatment of '~'
@@ -1687,9 +1696,8 @@ sub untaint_path {
     ($path) = ( $path =~ /(.*)/ );
     # win32 uses ';' for a path separator, assume others use ':'
     my $sep = WIN32 ? ';' : ':';
-    # -T disallows relative directories in the PATH
-    $path = join $sep, grep !/^\./, split /$sep/, $path;
-    return $path;
+    # -T disallows relative and empty directories in the PATH
+    return join $sep, grep !/^(\.|$)/, split /$sep/, $path;
 }
 
 sub pop_dir {
@@ -1856,6 +1864,44 @@ sub custom_config_path {
     return '';
 }
 
+# tries to nuke all occurences of custom config
+# used by things outside the A-T test suite
+sub custom_config_nuke {
+    my $cwd = fastcwd();
+
+    # 1) create a fake empty (blib/)?lib/Apache/TestConfigData.pm
+    # (don't delete it since it may mess up with MakeMaker)
+    my $path = catfile $cwd, "lib", Apache::TestConfig::CUSTOM_CONFIG_FILE;
+    # overwrite the empty stub
+    Apache::TestConfig::custom_config_write($path, '') if -e $path;
+
+    $path = catfile $cwd, "blib", "lib",
+        Apache::TestConfig::CUSTOM_CONFIG_FILE;
+    if (-e $path) {
+        my $mode = (stat _)[2];
+        my $mode_new = $mode | 0200;
+        chmod $mode_new, $path;
+        debug  "emptying $path";
+        Apache::TestConfig::custom_config_write($path, '');
+        chmod $mode, $path;
+    }
+
+    # 2) go through @INC = ~/.apache-test and nuke any occurences of
+    #    CUSTOM_CONFIG_FILE
+    my @inc  = ();
+
+    push @inc, catdir $ENV{HOME}, '.apache-test' if $ENV{HOME};
+
+    push @inc, @INC;
+
+    for (@inc) {
+        my $victim = File::Spec->rel2abs(catfile $_, CUSTOM_CONFIG_FILE);
+        next unless -e $victim;
+        debug "unlinking $victim";
+        unlink $victim;
+    }
+}
+
 sub custom_config_exists {
     # try to load custom config if it wasn't loaded yet (there are
     # many entry points to this API)
@@ -2105,6 +2151,12 @@ sub custom_config_load {
         return;
     }
 
+    if ($ENV{APACHE_TEST_INTERACTIVE_CONFIG_TEST}) {
+        debug "APACHE_TEST_INTERACTIVE_CONFIG_TEST=1 => " .
+            "skipping load of custom config data";
+        return;
+    }
+
     return if $custom_config_loaded;
 
     if (my $custom_config_path = custom_config_path()) {
@@ -2121,8 +2173,11 @@ sub custom_config_first_time {
     my $self = shift;
     my $conf_opts = shift;
 
-    unless (-t STDIN) {
-        error "STDIN is closed, can't run interactive config";
+    # we can't prompt when STDIN is not attached to tty, unless we
+    # were told that's it OK via env var (in which case some program
+    # will feed the interactive prompts
+    unless (-t STDIN || $ENV{APACHE_TEST_INTERACTIVE_PROMPT_OK}) {
+        error "STDIN is not attached to tty, skip interactive config";
         Apache::TestRun::skip_test_suite();
     }
 
@@ -2155,8 +2210,8 @@ If for some reason you want to skip the test suite, type: skip
     {
         my %choices = ();
         my @tries = qw(httpd httpd2);
-        # Win32 uses Apache or perhaps Apache2, not apache/apache2
-        push @tries, WIN32 ? qw(Apache Apache2) : qw(apache apache2);
+        # Win32 uses Apache not apache
+        push @tries, WIN32 ? qw(Apache) : qw(apache);
         for (grep defined $_,
              map({ catfile $vars->{$_}, $vars->{target} } qw(sbindir bindir)),
              $self->default_httpd, which($vars->{target}),
@@ -2215,7 +2270,7 @@ or via the environment variable APACHE_TEST_APXS. For example:
     # we probably could reconfigure on the fly ($self->configure), but
     # the problem is various cached data which won't be refreshed. so
     # the simplest is just to restart the run from scratch
-    Apache::TestRun::rerun();
+    Apache::TestRun::rerun($vars);
 }
 
 sub _custom_config_prompt_path {
@@ -2423,6 +2478,16 @@ configuration phase (C<t/TEST -config>, C<Apache::Test> will
 automatically unshift the I<project/lib> directory into C<@INC>, via
 the autogenerated I<t/conf/modperl_inc.pl> file.
 
+=head2 APACHE_TEST_INTERACTIVE_PROMPT_OK
+
+Normally interactive prompts aren't run when STDIN is not attached to
+a tty. But sometimes there is a program that can answer the prompts
+(e.g. when testing A-T itself). If this variable is true the
+interactive config won't be skipped (if needed).
+
+
+
+
 =head1 AUTHOR
 
 =head1 SEE ALSO
